/*
 * zgetdump - Tool for copying and converting System z dumps
 *
 * vmcoreinfo access functions
 *
 * Copyright IBM Corp. 2011
 * Author(s): Michael Holzheu <holzheu@linux.vnet.ibm.com>
 */

#include <elf.h>
#include "zgetdump.h"

#ifdef __s390x__
#define LC_VMCORE_INFO		0xe0c
#else
#define LC_VMCORE_INFO		0xe08
#endif

/*
 * File local static data
 */
static struct {
	char		*vmcoreinfo;
} l;

/*
 * Initialize vmcoreinfo
 */
void dfi_vmcoreinfo_init(void)
{
	unsigned long vmcoreinfo_addr;
	char vmcoreinfo_str[128];
	Elf64_Nhdr note;

	dfi_mem_read(LC_VMCORE_INFO, &vmcoreinfo_addr, sizeof(vmcoreinfo_addr));
	if (vmcoreinfo_addr == 0)
		return;
	if (dfi_mem_read_rc(vmcoreinfo_addr, &note, sizeof(note)))
		return;
	memset(vmcoreinfo_str, 0, sizeof(vmcoreinfo_str));
	if (dfi_mem_read_rc(vmcoreinfo_addr + sizeof(note), vmcoreinfo_str,
			    note.n_namesz))
		return;
	if (memcmp(vmcoreinfo_str, "VMCOREINFO", sizeof("VMCOREINFO") != 0))
		return;
	l.vmcoreinfo = zg_alloc(note.n_descsz + 1);
	if (dfi_mem_read_rc(vmcoreinfo_addr + 24, l.vmcoreinfo,
			    note.n_descsz)) {
		zg_free(l.vmcoreinfo);
		l.vmcoreinfo = NULL;
		return;
	}
	l.vmcoreinfo[note.n_descsz] = 0;
}

/*
 * Return vmcoreinfo data
 */
char *dfi_vmcoreinfo_get(void)
{
	return l.vmcoreinfo;
}

/*
 * Generic function: Return vmcoreinfo item (0 on failure)
 */
static u64 vmcoreinfo_item(const char *fmt, const char *sym, unsigned long base)
{

	char str[1024], *sym_str, *sym_str_end;
	u64 value;

	if (!l.vmcoreinfo)
		return 0;
	if (fmt)
		snprintf(str, sizeof(str), "%s(%s)=", fmt, sym);
	else
		snprintf(str, sizeof(str), "%s=", sym);
	sym_str = strstr(l.vmcoreinfo, str);
	if (!sym_str)
		return 0;
	sym_str += strlen(str);
	sym_str_end = strchr(sym_str, '\n');
	if (!sym_str_end)
		sym_str_end = strchr(sym_str, '\0');
	memset(str, 0, sizeof(str));
	memcpy(str, sym_str, (unsigned long) (sym_str_end - sym_str));
	value = strtoul(str, NULL, base);
	return value;
}

/*
 * Return vmcoreinfo symbol address (0 on failure)
 */
u64 dfi_vmcoreinfo_symbol(const char *sym)
{
	return vmcoreinfo_item("SYMBOL", sym, 16);
}

/*
 * Return vmcoreinfo symbol length (0 on failure)
 */
u64 dfi_vmcoreinfo_length(const char *sym)
{
	return vmcoreinfo_item("LENGTH", sym, 10);
}

/*
 * Return vmcoreinfo number (0 on failure)
 */
u64 dfi_vmcoreinfo_val(const char *sym)
{
	return vmcoreinfo_item(NULL, sym, 10);
}
